import re

import numpy as np
from pandas import Series

from hieroskopia.utils.evaluator import Evaluator


class InferNumeric:
    """
    Receive a column and try to analyze  the three digit separator,
    the decimal separator and get the numeric format pattern
    using regexp
    Return a dict with key named 'format'
    with the pandas patterns
    """

    @staticmethod
    def infer(series: Series):
        # Identify stage
        numeric_dict = {
            # ($)(+-) 1234 or 1234.12
            r'^([+-]?([0-9]*)(\.([0-9]+))?)$': {
                'three_digit_separator': '',
                'decimal_separator': '.'
            },
            # ($)(+-) 12,345 or 12,345.1234
            "^[+-]?\\d{1,3}(,\\d{3})*(\\.\\d+)?$": {
                'three_digit_separator': ',',
                'decimal_separator': '.'
            }
        }

        simple_int_pattern = r'[-+]?\d+'
        chars_not_allowed = r'[^\d,.\-\+]'

        # Trim whitespaces and currencies, add 0 if start with .
        series = series.astype(str).str.replace(r'[\$€£¥ ]', '', regex=True)
        series = Series(np.array(
            [re.sub(r'^(?=.)([+-]?)(\.)([0-9]*)$', '\\g<1>0\\g<2>\\g<3>', str(s)) for s in
             series.values])) if Evaluator(
            series).series_contains(r'^(?=.)([+-]?)(\.)([0-9]*)$') else series

        # If the series contains any number and not have any invalid character
        if Evaluator(series).series_match(simple_int_pattern) and not Evaluator(series).series_match(chars_not_allowed):
            # Validate each regular expression into the series
            for (re_exp, numeric_format) in numeric_dict.items():
                # If any regexp match, save this as its format
                if Evaluator(series).series_match(re_exp):
                    format_result = numeric_format
                    # Classify as float if the series have the decimal separator found in the regexp
                    format_result.update(type='float') if Evaluator(series).series_contains('\\' + format_result.get(
                        'decimal_separator')) else format_result.update(type='integer')
                    return format_result
        return {}



import pandas as pd

from hieroskopia import InferNumeric


class TestNumericAnalyser:
    @staticmethod
    def numeric_analyser_unittest(expected_list: list, data: dict):
        # Create test data frame
        df = pd.DataFrame(data)
        # Result list with format dicts
        result_list = []
        # Loop every df columns
        for col in df.columns:
            # Infer data format
            result_list.append(InferNumeric.infer(df[col]))
        # Print lists
        print('\n', '  Result list:', result_list, '\n', 'Expected list:',
              expected_list)
        # Test lists
        if expected_list != result_list:
            raise AssertionError

    def test_numeric_analyser(self):
        """
        Bad argument:
        Special logic:
        Boundary Values:
        """
        self.numeric_analyser_unittest(expected_list=[{},
                                                      {},
                                                      {'three_digit_separator': '', 'decimal_separator': '.',
                                                       'type': 'integer'},
                                                      {'three_digit_separator': ',', 'decimal_separator': '.',
                                                       'type': 'float'}],
                                       data={
                                           "date": ["2019-11-27",
                                                    "2019-11-28",
                                                    "2019-11-29"],
                                           "gateway": ["PROSA", "PROSA", "PROSA"],
                                           "amount": ["$4591", "$4592", "-$5"],
                                           "order_id": ['767,313,628,196.2', '76,731,362,819', '767,313,628,196']
                                       })



import pandas as pd

from hieroskopia import InferDatetime


class TestDateAnalyser:
    @staticmethod
    # Todo: Re use this fragment
    def date_analyser_unittest(expected_list: list, data: dict):
        # Create test data frame
        df = pd.DataFrame(data)
        # Result list with format dicts
        result_list = []
        # Loop every df columns
        for col in df.columns:
            # Infer data format
            result_list.append(InferDatetime.infer(df[col]))
        # Print lists
        print('\n', '  Result list:', result_list, '\n', 'Expected list:',
              expected_list)
        # Test lists
        if expected_list != result_list:
            raise AssertionError

    def test_date_analyser(self):
        """
        Bad argument:
        Special logic:
        Boundary Values:
        """
        self.date_analyser_unittest(expected_list=[{'formats': ['%Y-%m-%d', '%Y/%m/%d'], 'type': 'datetime'}, {}, {}, {}], data={
            "date": ["2019-11-27",
                     "2019/11/28",
                     "2018-11-08"],
            "gateway": ["PROSA", "PROSA", "PROSA"],
            "amount": ["$4591", "$4592", "$4593"],
            "order_id": [767313628196, 767313628196, 767313628196]
        })
